// set.cpp
//
// Copyright (C) 2006 Visual Media FX Ltd.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.

#include "set.hpp"

namespace olib { namespace openassetlib { namespace al {

set_ptr set::create()
{
	return set_ptr(new set());
}

set_ptr set::create(const asset_ptr& asset)
{
	return set_ptr(new set(asset));
}

set_ptr set::create(const set_ptr& s, bool deepcopy)
{
	return set_ptr(new set(s, deepcopy));
}

set::set(void)
{
}

set::set(const asset_ptr asset)
{
	if(asset == asset_ptr())
		return;

//	vec_.push_back(asset::create(asset));
	vec_.push_back(asset);
}

set::set(const set_ptr set, bool deepcopy)
{
	if(set == set_ptr())
		return;

	// copy of all assets into new set
	if(!set->empty())
	{
		const_iterator cit = set->begin();
		do
		{
			if(deepcopy)
				vec_.push_back(asset::create(*cit));
			else
				vec_.push_back(*cit);

		}	while(++cit != set->end());
	}
}

set::~set(void)
{
	vec_.clear();
}

void set::remove(const asset_ptr& asset)
{
	if(asset == asset_ptr())
		return;

	iterator it = begin();
	while(it != end())
	{
		if(asset->guid().compare((*it)->guid()) == 0)
			it = vec_.erase(it);
		else
			it++;
	}
}

bool set::insert(const asset_ptr& asset, int offset)
{
	if(asset == asset_ptr())
		return false;

	if((size_type)offset > size())
		return false;

	iterator it = begin();
	it += offset;
	insert(it, asset);
	return true;
}

bool set::insert(const set_ptr& set, int offset)
{
	if(set == set_ptr())
		return false;

	if((size_type)offset > size())
		return false;

	iterator it = begin();
	it += offset;
	insert(it, set->begin(), set->end());
	return true;
}

asset_ptr set::find(const opl::string& guid)
{
	for(iterator it = begin(); it != end(); it++)
	{
		if((*it)->guid().compare(guid) == 0)
			return (*it);
	}

	return asset_ptr();
}

} } }
