
// fx - A compositing library representation.

// Copyright (C) 2005-2006 Visual Media FX Ltd.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#define _WIN32_DCOM
#include <windows.h>
#include <objbase.h>
#else
#include <libxml/parser.h>
#endif

#include <openpluginlib/pl/utf8_utils.hpp>

#ifdef WIN32
#include <openeffectslib/fx/content_handler_msxml.hpp>
#else
#include <openeffectslib/fx/content_handler_libxml.hpp>
#endif

#include <openeffectslib/fx/n_operator_gpu_kernel_importer.hpp>

namespace fs  = boost::filesystem;
namespace opl = olib::openpluginlib;

namespace olib { namespace openeffectslib { namespace fx {

n_operator_gpu_kernel_importer::n_operator_gpu_kernel_importer( )
{
#ifdef WIN32
	CoInitializeEx( NULL, COINIT_SPEED_OVER_MEMORY | COINIT_APARTMENTTHREADED );
#endif
}

n_operator_gpu_kernel_importer::~n_operator_gpu_kernel_importer( )
{
#ifdef WIN32
	CoUninitialize( );
#endif
}

bool n_operator_gpu_kernel_importer::operator( )( const fs::path& file )
{	
#ifdef WIN32
	MSXML2::ISAXXMLReaderPtr pSAXReader = NULL;
	HRESULT hr = pSAXReader.CreateInstance( __uuidof( MSXML2::SAXXMLReader40 ) );

	assert( SUCCEEDED( hr ) && L"openeffectslib::n_operator_gpu_kernel_importer::operator( ) SAXReader instantiation failed." );
	
	content_handler_msxml* ch = new content_handler_msxml( );
	ch->set_branch_path( file.branch_path( ) );

	hr = pSAXReader->putContentHandler( ch );
	assert( SUCCEEDED( hr && L"openeffectslib::fx::operator( ) putContentHandler failed." ) );

	hr = pSAXReader->parseURL( ( unsigned short* ) opl::to_wstring( file.native_file_string( ).c_str( ) ).c_str( ) );
	assert( SUCCEEDED( hr ) && L"openeffectslib::fx::operator( ) parseURL failed." );
#else
	std::auto_ptr<content_handler_libxml> ch( new content_handler_libxml );
	ch->set_branch_path( file.branch_path( ) );

	if( xmlSAXUserParseFile( ch->get_content_handler( ), &ch->get_action( ), file.native_file_string( ).c_str( ) ) < 0 )
		; // we need log functionality!
#endif

	kernel = ch->get_action( ).kernel( );

	return true;
}

} } }
