
// fx - A compositing library representation.

// Copyright (C) 2005-2006 Visual Media FX Ltd.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.

#ifndef N_OPERATOR_GPU_KERNEL_PARSER_ACTION_INC_
#define N_OPERATOR_GPU_KERNEL_PARSER_ACTION_INC_

#if _MSC_VER >= 1400
#include <hash_map>
#endif

#include <map>
#include <vector>

#include <boost/filesystem/path.hpp>

#include <openpluginlib/pl/string.hpp>

// forward declarations
#ifdef WIN32
struct MSXML2::ISAXAttributes;
#else
#include <libxml/parser.h>
#endif // WIN32

namespace olib { namespace openeffectslib { namespace fx {

struct gpu_kernel_language
{
	explicit gpu_kernel_language( )
		: url( L"" )
	{ }

	openpluginlib::wstring url;
};

struct gpu_kernel_param
{
	explicit gpu_kernel_param( )
		: name( L"" )
		, type( L"" )
		, value( L"" )
	{ }
	
	openpluginlib::wstring name;
	openpluginlib::wstring type;
	openpluginlib::wstring value;
};

struct gpu_kernel_item
{
	typedef std::vector<gpu_kernel_param>::const_iterator const_iterator;
	
	explicit gpu_kernel_item( )
		: name( L"" )
		, GLSL_valid( false )
		, Cg_valid( false )
	{ }
	
	openpluginlib::wstring			name;
	gpu_kernel_language				GLSL_language;
	gpu_kernel_language				Cg_language;
	std::vector<gpu_kernel_param>	param;
	
	bool GLSL_valid;
	bool Cg_valid;
};

class n_operator_gpu_kernel_parser_action;

typedef bool ( *gpu_kernel_dispatcher )( n_operator_gpu_kernel_parser_action& pa );

class n_operator_gpu_kernel_parser_action
{
public:
	typedef openpluginlib::wstring 	key_type;
	typedef boost::filesystem::path	path;
	
#if _MSC_VER >= 1400
	typedef stdext::hash_multimap<key_type, gpu_kernel_dispatcher> gpu_kernel_dispatcher_container;
#else
	typedef std::multimap<key_type, gpu_kernel_dispatcher> gpu_kernel_dispatcher_container;
#endif

	explicit n_operator_gpu_kernel_parser_action( );
	virtual ~n_operator_gpu_kernel_parser_action( );

	bool dispatch( const openpluginlib::wstring& tag );

	void start( );
	void finish( );

	path get_branch_path( ) const;
	void set_branch_path( const path& branch_path );

	const gpu_kernel_item& kernel( ) const
	{ return kernel_; }
	gpu_kernel_item& kernel( )
	{ return kernel_; }

public:
#ifdef WIN32
	void set_attrs( MSXML2::ISAXAttributes __RPC_FAR* attrs );
#else
	void set_attrs( xmlChar** attrs );
#endif

public:
#ifdef WIN32
	MSXML2::ISAXAttributes __RPC_FAR* attrs_;
#else
	xmlChar** attrs_;
#endif

private:
	gpu_kernel_dispatcher_container dispatch_;
	path branch_path_;
	
private:
	gpu_kernel_item kernel_;
};

bool default_gpu_kernel_parser_action( n_operator_gpu_kernel_parser_action& pa );
bool kernel_gpu_kernel_parser_action( n_operator_gpu_kernel_parser_action& pa );
bool language_gpu_kernel_parser_action( n_operator_gpu_kernel_parser_action& pa );
bool param_gpu_kernel_parser_action( n_operator_gpu_kernel_parser_action& pa );

} } }

#endif
