
// fx - A compositing library representation.

// Copyright (C) 2005-2006 Visual Media FX Ltd.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.

#ifndef MEDIA_SOURCE_OPERATOR_INC_
#define MEDIA_SOURCE_OPERATOR_INC_

#if _MSC_VER
#pragma warning ( push )
#	pragma warning ( disable:4275 )
#endif

#include <GL/glew.h>

#include <openmedialib/ml/ml.hpp>

#include <openeffectslib/plugins/openmedialib_source/config.hpp>

#include <openeffectslib/fx/n_operator_gpu.hpp>
#include <openeffectslib/fx/playable.hpp>

namespace olib { namespace openeffectslib { namespace plugins {

//
class OPENMEDIALIB_SOURCE_DECLSPEC media_source_operator : public fx::n_operator_gpu, public fx::playable
{
private:
	typedef fx::v_frame_position::value_type frame_position_t;

public:
	explicit media_source_operator( );

	virtual bool init( );
	virtual bool update_image( );
	virtual bool update_audio( );
	virtual bool dirty( ) const { return true; }

	virtual int length( ) const;
	virtual double fps( ) const;
	virtual int position( ) const;
	virtual int position_image( ) const;
	virtual int position_audio( ) const;
	virtual void seek( int position, bool relative = false );
	virtual void seek_image( int position, bool relative = false );
	virtual void seek_audio( int position, bool relative = false );

private:
	bool fetch_frame( openmedialib::ml::input_type_ptr input, frame_position_t position );

private:
	openmedialib::ml::input_type_ptr input_;
	GLuint id_;
	GLenum target_;
	int old_phy_w_, old_phy_h_;

	fx::v_frame_position *frame_count_;
	openpluginlib::v_double *frame_rate_;
	fx::v_frame_position *image_position_;
	fx::v_frame_position *audio_position_;
};

} } }

#if _MSC_VER
#	pragma warning ( default:4275 )
#pragma warning ( pop )
#endif

#endif

