
// ml - A media library representation.

// Copyright (C) 2005 Visual Media FX Ltd.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.

#include <openmedialib/ml/ml.hpp>
#include <openmedialib/ml/openmedialib_plugin.hpp>
#include <openpluginlib/pl/openpluginlib.hpp>
#include <openpluginlib/pl/utf8_utils.hpp>

namespace ml = olib::openmedialib::ml;
namespace pl = olib::openpluginlib;

using namespace ml;

// Query structure used
struct ml_query_traits : public pl::default_query_traits
{
	ml_query_traits( const pl::wstring& filename, const pl::wstring &type )
		: filename_( filename )
		, type_( type )
	{ }
		
	pl::wstring to_match( ) const
	{ return filename_; }

	pl::wstring libname( ) const
	{ return pl::wstring( L"openmedialib" ); }

	pl::wstring type( ) const
	{ return pl::wstring( type_ ); }
	
	const pl::wstring filename_;
	const pl::wstring type_;
};

// Retrieve the first plugin which matches the resource and type (input or output)
static openmedialib_plugin_ptr get_plug( const pl::wstring &resource, const pl::wstring type )
{
	typedef pl::discovery< ml_query_traits > discovery;
	openmedialib_plugin_ptr result = openmedialib_plugin_ptr( );
	ml_query_traits query( resource, type );
	discovery plugins( query );
	if ( plugins.size( ) == 0 ) return result;
	discovery::const_iterator i = plugins.begin( );
	return boost::shared_dynamic_cast<ml::openmedialib_plugin>( i->create_plugin( "" ) );
}

// Return the first matching input object
ML_DECLSPEC input_type_ptr ml::create_input( const pl::wstring &resource )
{
	input_type_ptr result = input_type_ptr( );
	openmedialib_plugin_ptr plug = get_plug( resource, L"input" );
	return plug == 0 ? result : plug->input( resource );
}

// Return the first matching input object
ML_DECLSPEC input_type_ptr ml::create_input( const pl::string &resource )
{
	return create_input( pl::to_wstring( resource ) );
}

// Return the first matching store object
ML_DECLSPEC store_type_ptr ml::create_store( const pl::wstring &resource, frame_type_ptr frame )
{
	store_type_ptr result = store_type_ptr( );
	openmedialib_plugin_ptr plug = get_plug( resource, L"output" );
	return plug == 0 ? result : plug->store( resource, frame );
}

// Return the first matching store object
ML_DECLSPEC store_type_ptr ml::create_store( const pl::string &resource, frame_type_ptr frame )
{
	return create_store( pl::to_wstring( resource ), frame );
}

