
// avformat - A avformat plugin to ml.

// Copyright (C) 2005 Visual Media FX Ltd.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.




//encoding video parameters: width, height, and bitrate
#define WIDTH_ENCODED  1000//final window width
#define HEIGHT_ENCODED 1000//final window height

#define NUM_SAR2 4//sar2 numerator
#define DEN_SAR2 3//sar2 denominator

#define NUM_SAR1 16//sar1 numerator
#define DEN_SAR1 9//sar1 denominator

#define BIT_RATE_ENCODED 1024

//encoding audio parameters: bitrate input and output
#define BIT_RATE_AUDIOIN 127000
#define BIT_RATE_AUDIOOUT 100000






#include <openmedialib/ml/openmedialib_plugin.hpp>
#include <openpluginlib/pl/utf8_utils.hpp>

#ifdef WIN32
#include <windows.h>
#endif // WIN32


/* 5 seconds stream duration */
//#define STREAM_DURATION   60//3600.0 
#define STREAM_FRAME_RATE 25 /* 25 images/s */
//#define STREAM_NB_FRAMES  ((int)(STREAM_DURATION * STREAM_FRAME_RATE))
#define STREAM_PIX_FMT PIX_FMT_YUV420P /* default pix_fmt */


#include <iostream>
#include <cstdlib>
#include <vector>
#include <deque>
#include <string>
#include <cmath>
#include <cstdio>

#include <boost/thread/thread.hpp>
#include <boost/thread/condition.hpp>
#include <boost/thread/recursive_mutex.hpp>


extern "C" {
#include <avformat.h>
}


namespace opl = olib::openpluginlib;
namespace plugin = olib::openmedialib::ml;
namespace il = olib::openimagelib::il;

namespace olib { namespace openmedialib { namespace ml { 


char filenameSrc[1024];

	
class ML_PLUGIN_DECLSPEC avformat_store : public store_type, public store_keyboard_feedback
{

	AVStream *audio_st, *video_st;
	AVFormatContext *oc;
	AVOutputFormat *fmt;
    double audio_pts, video_pts;
	float t;
	float tincr, tincr2;
	int16_t *samples;
	int16_t *samples_resampled;

	uint8_t *audio_outbuf;
	//uint8_t * audio_outbuf_resampled;

	int audio_outbuf_size;
	//int audio_outbuf_size_resampled;

	int audio_input_frame_size;
	int frame_count;
	unsigned char * video_outbuf;
	int video_outbuf_size;

	AVFrame *picture;
	AVFrame * tmp_picture;

	ReSampleContext * context;

	int OffsetW, OffsetH;




private:
	avformat_store( frame_type_ptr frame, const char * filename=0 ) 
			: store_type( )
			, store_keyboard_feedback( )
			, WIN_WIDTH( 0 )
			, WIN_HEIGHT( 0 )
			, duration( 10 )
			, keyboard_handler_( 0 )
			, done_( false )
{

			frame_count=0;


			duration = int( frame->get_duration( ) * 1000 );
			frames_.push_back( frame );

			fmt = guess_format(NULL, ".mpg", NULL);
			if (!fmt) 
			{
				fprintf(stderr, "Could not find suitable output format\n");
				exit(1);
			}


			oc = av_alloc_format_context();
			if (!oc) 
			{
				fprintf(stderr, "Memory error\n");
				exit(1);
			}
			oc->oformat = fmt;


			/* add the audio and video streams using the default format codecs
			and initialize the codecs */
			video_st = NULL;
			audio_st = NULL;
			if (fmt->video_codec != CODEC_ID_NONE) 
				video_st = add_video_stream(oc, fmt->video_codec);

			if (fmt->audio_codec != CODEC_ID_NONE)
				audio_st = add_audio_stream(oc, fmt->audio_codec);



			/* set the output parameters (must be done even if no parameters). */
			if (av_set_parameters(oc, NULL) < 0) 
			{
				fprintf(stderr, "Invalid output format parameters\n");
				exit(1);
			}

			dump_format(oc, 0, filenameSrc, 1);

			/* now that all the parameters are set, we can open the audio and
			video codecs and allocate the necessary encode buffers */
			if (video_st)
				open_video(oc, video_st);
			if (audio_st)
				open_audio(oc, audio_st);

			/* open the output file, if needed */
			if (!(fmt->flags & AVFMT_NOFILE)) 
			{
				if (url_fopen(&oc->pb, filenameSrc, URL_WRONLY) < 0) 
				{
					fprintf(stderr, "Could not open '%s'\n", filenameSrc);
					exit(1);
				}
			}
			

			/* write the stream header, if any */
			av_write_header(oc);

}


int writeFrames(frame_type_ptr f)
{



				if (audio_st)
					audio_pts = (double)audio_st->pts.val * audio_st->time_base.num / audio_st->time_base.den;
				else
					audio_pts = 0.0;

				if (video_st)
					video_pts = (double)video_st->pts.val * video_st->time_base.num / video_st->time_base.den;
				else
					video_pts = 0.0;



				if (audio_pts < video_pts)
					write_audio_frame(oc, audio_st, f);
				else 
					write_video_frame(oc, video_st, f);

				
				

				return 1;

}
void finaliseEncoding()
{

			/* close each codec */
			if (video_st)
				close_video(oc, video_st);
			if (audio_st)
				close_audio(oc, audio_st);

			/* write the trailer, if any */
			av_write_trailer(oc);

			/* free the streams */
			for(int i = 0; i < oc->nb_streams; i++) {
				av_freep(&oc->streams[i]->codec);
				av_freep(&oc->streams[i]);
			}

			if (!(fmt->flags & AVFMT_NOFILE)) {
				/* close the output file */
				url_fclose(&oc->pb);
			}

			/* free the stream */
			av_free(oc);

}


 AVFrame *alloc_picture(int pix_fmt, int width, int height)
{
    AVFrame *picture;
    uint8_t *picture_buf;
    int size;

    picture = avcodec_alloc_frame();
    if (!picture)
        return NULL;
    size = avpicture_get_size(pix_fmt, width, height);
    picture_buf = (uint8_t *)malloc(size*3);
    if (!picture_buf) {
        av_free(picture);
        return NULL;
    }
    avpicture_fill((AVPicture *)picture, picture_buf,
                   pix_fmt, width, height);
    return picture;
}

 void open_video(AVFormatContext *oc, AVStream *st)
{
    AVCodec *codec;
    AVCodecContext *c;

    c = st->codec;

    /* find the video encoder */
    codec = avcodec_find_encoder(c->codec_id);
    if (!codec) {
        fprintf(stderr, "codec not found\n");
        exit(1);
    }

    /* open the codec */
    if (avcodec_open(c, codec) < 0) {
        fprintf(stderr, "could not open codec\n");
        exit(1);
    }


    /* allocate the encoded raw picture */
    picture = alloc_picture(c->pix_fmt, c->width, c->height);
    if (!picture) {
        fprintf(stderr, "Could not allocate picture\n");
        exit(1);
    }

    video_outbuf = 0;
    if (!(oc->oformat->flags & AVFMT_RAWPICTURE)) {
        /* allocate output buffer */
        /* XXX: API change will be done */
        video_outbuf_size = c->width * c->height * 3;
        video_outbuf = (uint8_t *)malloc(video_outbuf_size);
    }

    /* if the output format is not YUV420P, then a temporary YUV420P
       picture is needed too. It is then converted to the required
       output format */
    tmp_picture = NULL;
    if (c->pix_fmt != PIX_FMT_YUV420P) {
        tmp_picture = alloc_picture(PIX_FMT_YUV420P, c->width, c->height);
        if (!tmp_picture) {
            fprintf(stderr, "Could not allocate temporary picture\n");
            exit(1);
        }
    }
}


 AVStream *add_audio_stream(AVFormatContext *oc, int codec_id)
{
    AVCodecContext *c;
    AVStream *st;

    st = av_new_stream(oc, 1);
    if (!st) {
        fprintf(stderr, "Could not alloc stream\n");
        exit(1);
    }

    c = st->codec;
    c->codec_id = static_cast<CodecID>(codec_id);
    c->codec_type = CODEC_TYPE_AUDIO;

    //c->bit_rate = 64000;
    //c->sample_rate = 44100;

	//assumes this sample and bit rate... 
	//TODO: resample audio !!
    c->bit_rate = 128000;
    c->sample_rate = 44100;
    c->channels = 2;

    return st;
}

void open_audio(AVFormatContext *oc, AVStream *st)
 {
    AVCodecContext *c;
    AVCodec *codec;

    c = st->codec;

    /* find the audio encoder */
    codec = avcodec_find_encoder(c->codec_id);
    if (!codec) {
        fprintf(stderr, "codec not found\n");
        exit(1);
    }

    /* open it */
    if (avcodec_open(c, codec) < 0) {
        fprintf(stderr, "could not open codec\n");
        exit(1);
    }

    /* init signal generator */
    t = 0;
    tincr = 2 * M_PI * 110.0 / c->sample_rate;
    /* increment frequency by 110 Hz per second */
    tincr2 = 2 * M_PI * 110.0 / c->sample_rate / c->sample_rate;

    audio_outbuf_size = 10000;
    audio_outbuf = (uint8_t *)malloc(audio_outbuf_size);

    //audio_outbuf_size_resampled = 10000;
    //audio_outbuf_resampled = (uint8_t *)malloc(audio_outbuf_size_resampled);


    /* ugly hack for PCM codecs (will be removed ASAP with new PCM
       support to compute the input frame size in samples */
    if (c->frame_size <= 1) {
        audio_input_frame_size = audio_outbuf_size / c->channels;
        switch(st->codec->codec_id) {
        case CODEC_ID_PCM_S16LE:
        case CODEC_ID_PCM_S16BE:
        case CODEC_ID_PCM_U16LE:
        case CODEC_ID_PCM_U16BE:
            audio_input_frame_size >>= 1;
            break;
        default:
            break;
        }
    } else {
        audio_input_frame_size = c->frame_size;
    }

	context=audio_resample_init(2, 2, BIT_RATE_AUDIOOUT, BIT_RATE_AUDIOIN);


    samples = (int16_t *)malloc(audio_input_frame_size * 2 * c->channels);
    samples_resampled = (int16_t *)malloc(audio_input_frame_size * 2 * c->channels * 10);


}

void get_audio_frame(int16_t *samples, int16_t *src, int frame_size, int nb_channels)
{

	int16_t * destSamples, * srcSamples;

	srcSamples=src;
	destSamples=samples;

	for (int j=0;j<frame_size;j++)
	{
        for(int i = 0; i < nb_channels; i++)
		{
			*destSamples++ = *srcSamples++;
		}
	}

}

 void write_audio_frame(AVFormatContext *oc, AVStream *st, frame_type_ptr f)
{
    AVCodecContext *c;
    AVPacket pkt;
	int16_t *buf_audio;
	int nSamples;
	

	


    av_init_packet(&pkt);

    c = st->codec;

	audio_type_ptr a;

	a=f->get_audio();

	buf_audio=(int16_t *)a->data();

    get_audio_frame(samples, buf_audio, audio_input_frame_size, c->channels);


	nSamples=a->samples();
	


	audio_resample(context, (short *)samples_resampled, (short *)samples, nSamples);


    pkt.size= avcodec_encode_audio(c, audio_outbuf, audio_outbuf_size, samples_resampled);

	//audio_resample(context, (short *)audio_outbuf_resampled, (short *)samples, nSamples);



    pkt.pts= av_rescale_q(c->coded_frame->pts, c->time_base, st->time_base);
    pkt.flags |= PKT_FLAG_KEY;
    pkt.stream_index= st->index;
    pkt.data= audio_outbuf;

    /* write the compressed frame in the media file */
    if (av_interleaved_write_frame(oc, &pkt) != 0) 
	{
        fprintf(stderr, "Error while writing audio frame\n");
        exit(1);
    }



	
}

 void close_audio(AVFormatContext *oc, AVStream *st)
{
    avcodec_close(st->codec);

    free(samples);

	//if (samples_resampled)
	//	free(samples_resampled);

    free(audio_outbuf);

	//free(audio_outbuf_resampled);

	audio_resample_close(context);

}


//scales the frame...
void scale_frame(AVFrame *pict, image_type_ptr i, int W1, int H1)
{


	int pitchDest1, pitchDest2, pitchDest3, pitchSrc1, pitchSrc2, pitchSrc3;
	unsigned char * b1, *b2, *b3, * b0, * b01, * b02;
	float factx,facty, factx2;


	b1=(unsigned char *)i->data(0);
	b2=(unsigned char *)i->data(1);
	b3=(unsigned char *)i->data(2);

	b0=pict->data[0];
	b01=pict->data[1];
	b02=pict->data[2];

	pitchSrc1=i->pitch(0);
	pitchSrc2=i->pitch(1);
	pitchSrc3=i->pitch(2);

	pitchDest1=pict->linesize[0];
	pitchDest2=pict->linesize[1];
	pitchDest3=pict->linesize[2];

	
	facty=(float)i->get_crop_h()/((float)H1 - (float)OffsetH*2);
	factx=(float)i->get_crop_w()/((float)W1 - (float)OffsetW*2);
	factx2=factx/2;
	


	if ((facty==1.0f) && (factx==1.0f))//same size, so just copy, don't resize
	{
		memcpy(b0, b1, H1*W1);
		memcpy(b01, b2, (H1/2)*(W1/2) );
		memcpy(b02, b3, (H1/2)*(W1/2) );
	}
	else  // resize it
	{
		int x1,t;

		//this is green, for testing... put 16, 128, 128 for black colour
		memset(b0, 117, H1*W1);
		memset(b01, 70, (H1/2)*(W1/2) );
		memset(b02, 54, (H1/2)*(W1/2) );


		//Y, Cb, and Cr 
		for(int y=OffsetH;y<H1-OffsetH;y++) 
		{

			for(int x=OffsetW;x<W1-OffsetW;x++) 
			{

				
				t=x/2;

				x1=(int)((float)(x-OffsetW) * factx);
				b0[x]	= b1[x1];	//Y

				x1/=2;
				t=x/2;

				b01[t]	= b2[x1];	//Cb
				b02[t]	= b3[x1];	//Cr


			}

			b0+=pitchDest1;
			b01=pict->data[1] + ((y/2)*pitchDest2);
			b02=pict->data[2] + ((y/2)*pitchDest3);

			b1=(unsigned char *)i->data(0) + (((int)((float)(y-OffsetH)*facty))  * pitchSrc1);
			b2=(unsigned char *)i->data(1) + (((int)((float)((y-OffsetH)/2)*facty))  * pitchSrc2);
			b3=(unsigned char *)i->data(2) + (((int)((float)((y-OffsetH)/2)*facty))  * pitchSrc3);

		}

	}


}

/* prepare a dummy image */
 void fill_yuv_image(AVFrame *pict, int frame_index, int width, int height)
{
	
    int x, y;
	
// Y 
    for(y=0;y<height;y++) {
        for(x=0;x<width;x++) {
            pict->data[0][y * pict->linesize[0] + x] = (uint8_t)(frame_index * 3);
        }
    }
	
    // Cb and Cr 
    for(y=0;y<height/2;y++) {
        for(x=0;x<width/2;x++) {
            pict->data[1][y * pict->linesize[1] + x] =  (uint8_t)(y + frame_index * 3);
            pict->data[2][y * pict->linesize[2] + x] =  (uint8_t)(x + frame_index * 3);
        }
    }


	
}

 void write_video_frame(AVFormatContext *oc, AVStream *st, frame_type_ptr f)
{
    int out_size, ret;
    AVCodecContext *c;

    c = st->codec;

    //if (frame_count >= STREAM_NB_FRAMES) 
	//{
        /* no more frame to compress. The codec has a latency of a few
           frames if using B frames, so we get the last frames by
           passing the same picture again */
    //} 
	//else 
	//{
        if (c->pix_fmt != PIX_FMT_YUV420P) 
		{
            /* as we only generate a YUV420P picture, we must convert it
               to the codec pixel format if needed */
            fill_yuv_image(tmp_picture, frame_count, c->width, c->height);
            img_convert((AVPicture *)picture, c->pix_fmt,
                        (AVPicture *)tmp_picture, PIX_FMT_YUV420P,
                        c->width, c->height);
        } 
		else 
		{

			image_type_ptr i;

			i=f->get_image();
			
			if (i!=0)
				scale_frame(picture, i, WIDTH_ENCODED , HEIGHT_ENCODED);//


        }
    //}


    if (oc->oformat->flags & AVFMT_RAWPICTURE) {
        /* raw video case. The API will change slightly in the near
           futur for that */
        AVPacket pkt;
        av_init_packet(&pkt);

        pkt.flags |= PKT_FLAG_KEY;
        pkt.stream_index= st->index;
        pkt.data= (uint8_t *)picture;
        pkt.size= sizeof(AVPicture);

        ret = av_write_frame(oc, &pkt);
    } else {
        /* encode the image */
        out_size = avcodec_encode_video(c, video_outbuf, video_outbuf_size, picture);
        /* if zero size, it means the image was buffered */
        if (out_size > 0) {
            AVPacket pkt;
            av_init_packet(&pkt);

            pkt.pts= av_rescale_q(c->coded_frame->pts, c->time_base, st->time_base);
            if(c->coded_frame->key_frame)
                pkt.flags |= PKT_FLAG_KEY;
            pkt.stream_index= st->index;
            pkt.data= video_outbuf;
            pkt.size= out_size;

            /* write the compressed frame in the media file */
            ret = av_interleaved_write_frame(oc, &pkt);
        } else {
            ret = 0;
        }
    }
    if (ret != 0) {
        fprintf(stderr, "Error while writing video frame\n");
        exit(1);
    }
    frame_count++;
}



/* add a video output stream */
 AVStream *add_video_stream(AVFormatContext *oc, int codec_id)
{
    AVCodecContext *c;
    AVStream *st;

    st = av_new_stream(oc, 0);
    if (!st) {
        fprintf(stderr, "Could not alloc stream\n");
        exit(1);
    }

    c = st->codec;
    c->codec_id = static_cast<CodecID>(codec_id);
    c->codec_type = CODEC_TYPE_VIDEO;


    c->bit_rate = BIT_RATE_ENCODED;



	c->width= WIDTH_ENCODED;
	c->height = HEIGHT_ENCODED;

	float r1,r2,nh,nw;


	r1=(float)NUM_SAR1 / (float)DEN_SAR1;
	r2=(float)NUM_SAR2 / (float)DEN_SAR2;

	//r1=1.3333f;
	
	if (r1==r2)
	{
		OffsetW=0;
		OffsetH=0;
	}
	if (r1<r2)
	{
		//nh=((float)DEN_SAR2 / (float)NUM_SAR2) * (float)WIDTH_ENCODED;
		nh=(r1/r2)*(float)HEIGHT_ENCODED;
		OffsetH=((float)HEIGHT_ENCODED - nh)/2.0f;
		OffsetW=0;
	}
	if (r1>r2)
	{
		//nw=((float)DEN_SAR2 / (float)NUM_SAR2) * (float)WIDTH_ENCODED;
		nw=(r2/r1)*(float)WIDTH_ENCODED;
		OffsetW=((float)WIDTH_ENCODED - nw)/2.0f;
		OffsetH=0;
	}



    /* time base: this is the fundamental unit of time (in seconds) in terms
       of which frame timestamps are represented. for fixed-fps content,
       timebase should be 1/framerate and timestamp increments should be
       identically 1. */
    c->time_base.den = STREAM_FRAME_RATE;
    c->time_base.num = 1;

	


    c->pix_fmt = STREAM_PIX_FMT;



    if (c->codec_id == CODEC_ID_MPEG2VIDEO) {
        /* just for testing, we also add B frames */
        c->max_b_frames = 2;
    }
    if (c->codec_id == CODEC_ID_MPEG1VIDEO){
        /* needed to avoid using macroblocks in which some coeffs overflow
           this doesnt happen with normal video, it just happens here as the
           motion of the chroma plane doesnt match the luma plane */
        c->mb_decision=2;
    }
    // some formats want stream headers to be seperate
    if(!strcmp(oc->oformat->name, "mp4") || !strcmp(oc->oformat->name, "mov") || !strcmp(oc->oformat->name, "3gp"))
        c->flags |= CODEC_FLAG_GLOBAL_HEADER;

	

    return st;
}



 void close_video(AVFormatContext *oc, AVStream *st)
{

    avcodec_close(st->codec);

	if (picture)
	{
		//av_free(picture->data[0]);
		av_free(picture);
	}
    if (tmp_picture) 
	{
        //av_free(tmp_picture->data[0]);
        av_free(tmp_picture);
    }
	if (video_outbuf)
		free(video_outbuf);

	//if (filenameSrc)
	//	free(filenameSrc);
}


	public:
		virtual ~avformat_store( )
		{
			//free(buff);
			audio_ = store_type_ptr( );
		}

		static avformat_store *get_instance( )
		{
			return instance_;
		}

		static avformat_store *get_instance( frame_type_ptr frame )
		{
			if ( instance_ == 0 )
			{
				instance_ = new avformat_store( frame, "silly test.mpg" );

			}
			return instance_;
		}


		frame_type_ptr get_frame( )
		{
			
			if ( frames_.size( ) == 0 ) return frame_type_ptr( );
			frame_type_ptr frame = frames_[ 0 ];
			return frame;
		}




		virtual void register_keyboard_handler( store_keyboard_handler *handler )
		{
			keyboard_handler_ = handler;
		}

		static void keyboard( unsigned char key, int /*x*/, int /*y*/ )
		{
			get_instance( )->inner_keyboard( key );
		}

		inline void inner_keyboard( unsigned char key )
		{
			if ( keyboard_handler_ != 0 )
				keyboard_handler_->keyboard_handler( key );
		}


		virtual bool push( frame_type_ptr frame )
		{

			if (audio_st!=0)
				writeFrames(frame);
			if (video_st!=0)
				writeFrames(frame);

			return true;

		}



		virtual frame_type_ptr flush( )
		{
			
			if ( audio_ )
				audio_->flush( );
			
			frame_type_ptr frame = frame_type_ptr( );
			if ( frames_.size( ) != 0 )
				frame = *frames_.begin( );
			frames_.clear( );
			
			return frame;
		}

		virtual void complete( )//termination of the encoding....
		{

			done_ = true;

			/* close each codec */
			if (video_st)
				close_video(oc, video_st);
			if (audio_st)
				close_audio(oc, audio_st);

			/* write the trailer, if any */
			av_write_trailer(oc);


			//notify( );
		}

	private:
		int WIN_WIDTH;
		int WIN_HEIGHT;
		int duration;
		//boost::mutex mutex_;
		static avformat_store *instance_;
		//boost::mutex cond_mutex_;
		//boost::condition cond_;
		store_type_ptr audio_;
		std::deque< ml::frame_type_ptr > frames_;
		//GLuint texture_id;
		ml::store_keyboard_handler *keyboard_handler_;
		bool done_;

		int width,height,pitch;
		unsigned char * buffer;
		

public:





};
avformat_store *avformat_store::instance_ = 0;

class ML_PLUGIN_DECLSPEC avformat_input : public input_type
{
	public:
		// Constructor and destructor
		avformat_input( opl::wstring resource, const opl::wstring mime_type = L"" ) 
			: input_type( ) 
			, uri_( resource )
			, mime_type_( mime_type )
			, frames_( 0 )
			, video_streams_( 0 )
			, audio_streams_( 0 )
			, context_( NULL )
			, format_( NULL )
			, params_( NULL )
			, video_index_( -1 )
			, audio_index_( -1 )
			, expected_( 0 )
			, video_codec_( 0 )
			, audio_codec_( 0 )
			, must_decode_( true )
			, key_search_( false )
			, audio_buf_used_( 0 )
			, audio_buf_offset_( 0 )
		{


			

			// Ugly - looking to see if a dv1394 device has been specified

						

			if ( resource.find( L"/dev/" ) == 0 && resource.find( L"1394/" ) != opl::wstring::npos )
				format_ = av_find_input_format( "dv" );

			// Allow dv on stdin
			if ( resource == L"dv:-" )
			{
				format_ = av_find_input_format( "dv" );
				resource = L"pipe:";
			}

			// Allow mpeg on stdin
			if ( resource == L"mpeg:-" )
			{
				format_ = av_find_input_format( "mpeg" );
				resource = L"pipe:";
			}

			// Attempt to open the resource
			int error = av_open_input_file( &context_, opl::to_string( resource ).c_str( ), format_, 0, params_ ) < 0;

			// Get the stream info
			if ( error == 0 )
				error = av_find_stream_info( context_ ) < 0;
			
			// Populate the input properties
			if ( error == 0 )
				populate( );

			// Allocate an av frame
			av_frame_ = avcodec_alloc_frame( );
		}

		virtual ~avformat_input( ) 
		{
			if ( video_index_ >= 0 )
				close_video_codec( );
			if ( audio_index_ >= 0 )
				close_audio_codec( );
			if ( context_ )
				av_close_input_file( context_ );
			av_free( av_frame_ );
		}

		bool is_valid( )
		{
			return context_ != 0;
		}

		// Analyse streams and set input values
		void populate( )
		{
			
			

			// Iterate through all the streams available
			for( int i = 0; i < context_->nb_streams; i++ ) 
			{
				// Get the codec context
   				AVCodecContext *codec_context = context_->streams[ i ]->codec;
		
				// Determine the type and obtain the first index of each type
   				switch( codec_context->codec_type ) 
				{
					case CODEC_TYPE_VIDEO:
						video_streams_ ++;
						if ( video_index_ < 0 )
						{
							video_index_ = i;
							width_ = codec_context->width;
							height_ = codec_context->height;

							//WidthSrc = 128;//width_;//width_ * 2;//stores the w and h, to be used in the store class...
							//HeightSrc = 128;//height_;//height_ * 2;



							//filenameSrc, context_->filename
							int size=strlen(context_->filename);
							//filenameSrc=(char *)malloc(sizeof(size)+1);
							memset(filenameSrc,0,size+1);
							sprintf(filenameSrc, "%s", context_->filename);
							memcpy(&filenameSrc[size-3],"mpg",3);//name to be used in the store class

							

						}
						break;
					case CODEC_TYPE_AUDIO:
						audio_streams_ ++;
						if ( audio_index_ < 0 )
							audio_index_ = i;
		   				break;
					default:
		   				break;
				}
			}

			// Configure video input properties
			if ( has_video( ) )
			{
				// TODO: Need to check this one
				if ( get_video_stream( )->r_frame_rate.num != 0 && get_video_stream( )->r_frame_rate.den != 0 )
				{
					fps_num_ = get_video_stream( )->r_frame_rate.num;
					fps_den_ = get_video_stream( )->r_frame_rate.den;
				}
				else
				{
					fps_num_ = get_video_stream( )->codec->time_base.den;
					fps_den_ = get_video_stream( )->codec->time_base.num;
				}
			}
			else if ( has_audio( ) )
			{
				fps_num_ = 25;
				fps_den_ = 1;
				sar_num_ = 1;
				sar_den_ = 1;
			}

			// Open the video and audio codecs
			if ( has_video( ) ) open_video_codec( );
			if ( has_audio( ) ) open_audio_codec( );

			// Set the duration
			if ( ( uint64_t )context_->duration != AV_NOPTS_VALUE )
				frames_ = int( ( input_type::fps( ) * ( double )context_->duration / ( double )AV_TIME_BASE ) - 2 );
			else
				frames_ = 1 << 29;

			// Work around for inefficiencies on I frame only seeking
			// - this should be covered by the AVStream discard and need_parsing values
			//   but they're either wrong or poorly documented...
			if ( has_video( ) )
			{
				std::string codec( get_video_stream( )->codec->codec->name );
				if ( codec == "mjpeg" )
					must_decode_ = false;
				else if ( codec == "dvvideo" )
					must_decode_ = false;
				else if ( codec == "cinepak" )
					must_decode_ = false;
			}
		}

		// Returns the current video stream
		inline AVStream *get_video_stream( )
		{
			return context_->streams[ video_index_ ];
		}

		// Returns the current audio stream
		inline AVStream *get_audio_stream( )
		{
			return context_->streams[ audio_index_ ];
		}

		// Opens the video codec associated to the current stream
		void open_video_codec( )
		{
			AVStream *stream = get_video_stream( );
			AVCodecContext *codec_context = stream->codec;
			video_codec_ = avcodec_find_decoder( codec_context->codec_id );
			if ( video_codec_ == NULL || avcodec_open( codec_context, video_codec_ ) < 0 )
				video_index_ = -1;
		}

		void close_video_codec( )
		{
			AVStream *stream = get_video_stream( );
			if ( stream && stream->codec )
				avcodec_close( stream->codec );
		}
		
		// Opens the audio codec associated to the current stream
		void open_audio_codec( )
		{
			AVStream *stream = get_audio_stream( );
			AVCodecContext *codec_context = stream->codec;
			audio_codec_ = avcodec_find_decoder( codec_context->codec_id );
			if ( audio_codec_ == NULL || avcodec_open( codec_context, audio_codec_ ) < 0 )
				audio_index_ = -1;
		}

		void close_audio_codec( )
		{
			AVStream *stream = get_audio_stream( );
			if ( stream && stream->codec )
				avcodec_close( stream->codec );
		}

		// Gets the timestamp of the requested frame
		inline int64_t get_time_stamp( )
		{
			int64_t timestamp = ( int64_t )( ( double )get_position( ) / input_type::fps( ) * AV_TIME_BASE );
			if ( ( uint64_t )context_->start_time != AV_NOPTS_VALUE )
				timestamp += context_->start_time;
			return timestamp;
		}

		// Seek to the requested frame
		inline bool seek_to_position( )
		{
			

			int64_t offset = get_time_stamp( );
			if ( must_decode_ )
			{
				offset -= AV_TIME_BASE;
				if ( offset < context_->start_time )
					offset = 0;
				key_search_ = true;
			}
			return av_seek_frame( context_, -1, offset, AVSEEK_FLAG_BACKWARD ) >= 0;
		}

		// Decode the image





		int decode_image( bool &got_picture )
		{
			
			AVCodecContext *codec_context = get_video_stream( )->codec;

			int ret = 0;
			int got_dummy = 0;
			int got_pict = 0;
			
			// Derive the pts
			double pts = 0;
			if ( ( uint64_t )pkt_.dts != AV_NOPTS_VALUE )
            	pts = av_q2d( get_video_stream( )->time_base ) * pkt_.dts;

			int position = int( pts * fps( ) + 0.5 );

			if ( ( uint64_t )context_->start_time != AV_NOPTS_VALUE )
				position -= int( context_->start_time * input_type::fps( ) / AV_TIME_BASE );

			if ( !must_decode_ && ( get_process_flags( ) & ml::process_image ) == 0 )
			{
				got_picture = true;
				return ret;
			}

			// We have to continue processing until we get a pts >= to the requested position
			if ( position >= get_position( ) )
				avcodec_decode_video( codec_context, av_frame_, &got_pict, pkt_.data, pkt_.size );
			else if ( must_decode_ )
				avcodec_decode_video( codec_context, av_frame_, &got_dummy, pkt_.data, pkt_.size );



			got_picture = got_pict != 0;

			// If we have just done a search, then we need to locate the first key frame
			// all others are discarded.
			if ( key_search_ && av_frame_->key_frame == 0 && position < get_position( ) )
				got_picture = got_dummy = false;
			else
				key_search_ = false;

			// Store the image in the queue
			if ( position >= 0 && ( got_picture || got_dummy ) )
				store_image( pts, position );

			return ret;
		}

		const std::wstring avformat_to_oil( int fmt )
		{
			if ( fmt == PIX_FMT_YUV420P )
				return L"yuv420p";
			else if ( fmt == PIX_FMT_YUVJ420P )
				return L"yuv420p";
			else if ( fmt == PIX_FMT_UYVY411 )
				return L"yuv411";
			else if ( fmt == PIX_FMT_YUV411P )
				return L"yuv411p";
			else if ( fmt == PIX_FMT_YUV422 )
				return L"yuv422";
			else if ( fmt == PIX_FMT_YUV422P )
				return L"yuv422p";
			else if ( fmt == PIX_FMT_YUVJ422P )
				return L"yuv422p";
			else if ( fmt == PIX_FMT_RGB24 )
				return L"r8g8b8";
			else if ( fmt == PIX_FMT_BGR24 )
				return L"b8g8r8";
			else if ( fmt == PIX_FMT_RGBA32 )
				return L"r8g8b8a8";
			return L"";
		}

		void store_image( double pts, int64_t position )
		{
			AVStream *stream = get_video_stream( );
			AVCodecContext *codec_context = stream->codec;
			int width = get_width( );
			int height = get_height( );
			std::wstring format = avformat_to_oil( codec_context->pix_fmt );
			image_type_ptr image;
			if ( format == L"" )
			{
				image = il::allocate( L"b8g8r8", width, height );
        		AVPicture output;
        		avpicture_fill( &output, image->data( ), PIX_FMT_BGR24, width, height );
        		img_convert( &output, PIX_FMT_BGR24, (AVPicture *)av_frame_, codec_context->pix_fmt, width, height );
			}
			else
			{
				image = il::allocate( format, width, height );
				AVPicture output;
				avpicture_fill( &output, image->data( ), codec_context->pix_fmt, width, height );
				img_convert( &output, codec_context->pix_fmt, (AVPicture *)av_frame_, codec_context->pix_fmt, width, height );
			}

			image->set_pts( pts );
			image->set_position( position );

			if ( images_.size( ) > 0 )
			{
				int first = images_[ 0 ]->position( );
				int last = images_[ images_.size( ) - 1 ]->position( );
				if ( position < last )
					images_.clear( );
				else if ( first < get_position( ) - 15 )
					images_.erase( images_.begin( ) );
			}

			if ( av_frame_->interlaced_frame )
				image->set_field_order( av_frame_->top_field_first ? il::top_field_first : il::bottom_field_first );

			image->set_writable( false );
			images_.push_back( image );
		}

		inline int samples_for_frame( int frequency, int index )
		{
			double fps = input_type::fps( );
			int samples = int( double( frequency ) / fps );
			int discrepancy = frequency - int( samples * fps );
			if ( discrepancy )
			{
				if ( index % int( fps ) == 1 )
					samples += discrepancy;
			}
			return samples;
		}

		int decode_audio( bool &got_audio )
		{
			int ret = 0;

			// Get the audio codec context
			AVCodecContext *codec_context = get_audio_stream( )->codec;

			// The number of bytes in the packet
			int len = pkt_.size;
			uint8_t *data = pkt_.data;

			// This is the pts of the packet
			double pts = 0;
			if ( ( uint64_t )pkt_.pts != AV_NOPTS_VALUE )
           		pts = av_q2d( get_audio_stream( )->time_base ) * pkt_.pts;
			int found = int( pts * input_type::fps( ) + 0.5 );

			if ( ( uint64_t )context_->start_time != AV_NOPTS_VALUE )
				found -= int( context_->start_time * input_type::fps( ) / AV_TIME_BASE );


			// Get the audio info from the codec context
			int channels = codec_context->channels;
			int frequency = codec_context->sample_rate;
			int bps = 2;

			if ( audio_buf_used_ == 0 )
			{
				if ( audio_.size( ) != 0 )
					audio_buf_offset_ = ( *( audio_.end( ) - 1 ) )->position( ) + 1;
				else
					audio_buf_offset_ = found;
			}

			// Each packet may need multiple parses
			while( len > 0 )
			{
				int audio_size = 0;

				// Decode the audio into the buffer
           		ret = avcodec_decode_audio( codec_context, ( int16_t * )( audio_buf_ + audio_buf_used_ ), &audio_size, data, len );

				// If no samples are returned, then break now
				if ( ret < 0 )
				{
					ret = 0;
					got_audio = true;
					audio_index_ = -1;
					audio_streams_ = 0;
					break;
				}

				// Decrement the length by the number of bytes parsed
				len -= ret;
				data += ret;

				// Increment the number of bytes used in the buffer
				if ( audio_size > 0 )
					audio_buf_used_ += audio_size;

				// Current index in the buffer
				int index = 0;

				// Now we need to split the audio up into frame sized dollops
				while( 1 )
				{
					int samples = samples_for_frame( frequency, audio_buf_offset_ );

					int bytes_for_frame = samples * channels * bps;

					if ( bytes_for_frame > ( audio_buf_used_ - index ) )
						break;

					// Create an audio frame
					index += store_audio( audio_buf_offset_, audio_buf_ + index, samples );

					if ( audio_buf_offset_ >= get_position( ) )
						got_audio = 1;

					audio_buf_offset_ += 1;
				}

				audio_buf_used_ -= index;
				if ( audio_buf_used_ && index != 0 )
					memmove( audio_buf_, audio_buf_ + index, audio_buf_used_ );
			}

			return ret;
		}

		int store_audio( int position, uint8_t *buf, int samples )
		{
			typedef audio< unsigned char, pcm16 > pcm16_audio_type;

			// Get the audio codec context
			AVCodecContext *codec_context = get_audio_stream( )->codec;

			// Get the audio info from the codec context
			int channels = codec_context->channels;
			int frequency = codec_context->sample_rate;

			audio_type_ptr aud = audio_type_ptr( new audio_type( pcm16_audio_type( frequency, channels, samples ) ) );
			aud->set_position( position );
			memcpy( aud->data( ), buf, aud->size( ) );

			if ( audio_.size( ) > 0 )
			{
				int first = audio_[ 0 ]->position( );
				int last = audio_[ audio_.size( ) - 1 ]->position( );
				if ( position != last + 1 )
					audio_.clear( );
				else if ( first < get_position( ) - 15 )
					audio_.erase( audio_.begin( ) );
			}

			audio_.push_back( aud );

			return aud->size( );
		}

		void clear_stores( bool force = false )
		{
			bool clear = true;

			if ( !force && images_.size( ) )
			{
				int current = get_position( );
				int first = images_[ 0 ]->position( );
				int last = images_[ images_.size( ) - 1 ]->position( );
				if ( current >= first && current <= last )
					clear = false;
			}

			if ( clear )
			{
				images_.clear( );
				audio_.clear( );
				audio_buf_used_ = 0;
			}
		}

		void find_image( frame_type *frame )
		{
			int current = get_position( );
			int closest = 1 << 16;
			std::deque< image_type_ptr >::iterator result = images_.end( );
			std::deque< image_type_ptr >::iterator iter;

			for ( iter = images_.begin( ); iter != images_.end( ); iter ++ )
			{
				image_type_ptr img = *iter;
				int diff = current - img->position( );
				if ( std::abs( diff ) <= closest )
				{
					result = iter;
					closest = std::abs( diff );
				}
				else if ( img->position( ) > current )
				{
					break;
				}
			}

			if ( result != images_.end( ) )
				frame->set_image( *result );
		}

		void find_audio( frame_type *frame )
		{
			int current = get_position( );
			int closest = 1 << 16;
			std::deque< audio_type_ptr >::iterator result = audio_.end( );
			std::deque< audio_type_ptr >::iterator iter;

			for ( iter = audio_.begin( ); iter != audio_.end( ); iter ++ )
			{
				audio_type_ptr aud = *iter;
				int diff = current - aud->position( );
				if ( std::abs( diff ) <= closest )
				{
					result = iter;
					closest = std::abs( diff );
				}
				else if ( aud->position( ) > current )
				{
					break;
				}
			}

			if ( result != audio_.end( ) )
			{
				frame->set_audio( *result );
				frame->set_duration( double( ( *result )->samples( ) ) / double( ( *result )->frequency( ) ) );
			}
		}

		// Basic information
		virtual const opl::wstring get_uri( ) const { return uri_; }
		virtual const opl::wstring get_mime_type( ) const { return mime_type_; }
		virtual bool has_video( ) const { return video_index_ != -1 && video_streams_ > 0; }
		virtual bool has_audio( ) const { return audio_index_ != -1 && audio_streams_ > 0; }

		// Audio/Visual
		virtual int get_frames( ) const { return frames_; }
		virtual bool is_seekable( ) const { return is_seekable_; }

		// Visual
		virtual void get_fps( int &num, int &den ) const { num = fps_num_; den = fps_den_; }
		virtual void get_sar( int &num, int &den ) const { num = sar_num_; den = sar_den_; }
		virtual int get_video_streams( ) const { return video_streams_; }
		virtual int get_width( ) const { return width_; }
		virtual int get_height( ) const { return height_; }

		// Audio
		virtual int get_audio_streams( ) const { return audio_streams_; }
		virtual bool set_video_stream( const int stream ) { video_index_ = stream; return true; }
		virtual bool set_audio_stream( const int stream ) { audio_index_ = stream; return true; }

		// Fetch method
		virtual frame_type_ptr fetch( )
		{
			int process_flags = get_process_flags( );

			// Seek to the correct position if necessary
			if ( get_position( ) != expected_ )
			{
				int valid = 0;
				if ( images_.size( ) > 0 )
					valid = images_[ 0 ]->position( );
				if ( audio_.size( ) > 0 )
					valid = audio_[ 0 ]->position( ) > valid ? audio_[ 0 ]->position( ) : valid;

				if ( get_position( ) >= valid && get_position( ) < int( expected_ + 2 * fps( ) ) )
					;
				else if ( seek_to_position( ) )
					clear_stores( true );
				else
					seek( expected_ );
				expected_ = get_position( );
			}

			// Clear the packet
			av_init_packet( &pkt_ );

			// Loop until an error or we have what we need
			int error = 0;
			bool got_picture = !has_video( );
			bool got_audio = !has_audio( );
			int current = get_position( );

			if ( ( process_flags & process_image ) != 0 && ( has_video( ) && images_.size( ) > 0 ) )
			{
				int first = images_[ 0 ]->position( );
				int last = images_[ images_.size( ) - 1 ]->position( );
				if ( current >= first && current <= last )
					got_picture = true;
			}

			if ( ( process_flags & process_audio ) != 0 && ( has_audio( ) && audio_.size( ) > 0 ) )
			{
				int first = audio_[ 0 ]->position( );
				int last = audio_[ audio_.size( ) - 1 ]->position( );
				if ( current >= first && current <= last )
					got_audio = true;
			}

			while( error >= 0 && ( !got_picture || !got_audio ) )
			{
				error = av_read_frame( context_, &pkt_ );
				if ( error >= 0 && pkt_.stream_index == video_index_ )
				{
					error = decode_image( got_picture );
				}
				else if ( error >= 0 && pkt_.stream_index == audio_index_ )
					error = decode_audio( got_audio );
				av_free_packet( &pkt_ );
			}

			// Hmmph
			if ( has_video( ) )
			{
				sar_num_ = get_video_stream( )->codec->sample_aspect_ratio.num;
				sar_den_ = get_video_stream( )->codec->sample_aspect_ratio.den;
			}

			// Create the output frame
			frame_type *result = new frame_type( );

			int num, den;
			get_sar( num, den );
			result->set_sar( num, den );
			result->set_position( expected_ );
			result->set_pts( expected_ * 1.0 / input_type::fps( ) );
			result->set_duration( 1.0 / input_type::fps( ) );

			if ( ( process_flags & process_image ) && has_video( ) )
				find_image( result );
			if ( ( process_flags & process_audio ) && has_audio( ) )
				find_audio( result );

			// Update the next expected position
			expected_ ++;

			return frame_type_ptr( result );
		}


	private:
		opl::wstring uri_;
		opl::wstring mime_type_;
		int frames_;
		bool is_seekable_;
		int fps_num_;
		int fps_den_;
		int sar_num_;
		int sar_den_;
		int video_streams_;
		int width_;
		int height_;
		int audio_streams_;

		AVFormatContext *context_;
		AVInputFormat *format_;
		AVFormatParameters *params_;
		int video_index_;
		int audio_index_;
		int expected_;
		AVFrame *av_frame_;
		AVCodec *video_codec_;
		AVCodec *audio_codec_;
		AVPacket pkt_;
		std::deque < image_type_ptr > images_;
		std::deque < audio_type_ptr > audio_;
		bool must_decode_;
		bool key_search_;

    	unsigned char audio_buf_[ ( AVCODEC_MAX_AUDIO_FRAME_SIZE * 3 ) / 2 ]; 
    	int audio_buf_used_;
		int audio_buf_offset_;
};

//
// Plugin object
//

class ML_PLUGIN_DECLSPEC avformat_plugin : public openmedialib_plugin
{
public:

	virtual input_type_ptr input( const opl::wstring &resource )
	{
 		typedef boost::shared_ptr< avformat_input > result_type_ptr;
		result_type_ptr result = result_type_ptr( new avformat_input( resource ) );
		if ( !result->is_valid( ) )
			result = result_type_ptr( );


		return result;
	}
	//este es el metodo que hay que hacer maana, para que se cree un store....
	//virtual store_type_ptr store( const openpluginlib::wstring &resource, const frame_type_ptr &frame ) = 0;




	virtual store_type_ptr store( const opl::wstring &, const frame_type_ptr &frame )
	{
		return store_type_ptr( avformat_store::get_instance( frame ));
		
		
	}
};

} } }

//
// Library initialisation mechanism
//

namespace
{
	void reflib( int init )
	{
		static long refs = 0;

		assert( refs >= 0 && L"avformat_plugin::refinit: refs is negative." );
		
		if( init > 0 && ++refs == 1)
		{
			av_register_all( );
			av_log_set_level( -1 );
		}
		else if( init < 0 && --refs == 0 )
		{
			av_free_static( );
		}
	}
	
	boost::recursive_mutex mutex;
}

//
// Access methods for openpluginlib
//

extern "C"
{
	ML_PLUGIN_DECLSPEC bool openplugin_init( void )
	{
		boost::recursive_mutex::scoped_lock lock( mutex );
		reflib( 1 );
		return true;
	}

	ML_PLUGIN_DECLSPEC bool openplugin_uninit( void )
	{
		boost::recursive_mutex::scoped_lock lock( mutex );
		reflib( -1 );
		return true;
	}
	
	ML_PLUGIN_DECLSPEC bool openplugin_create_plugin( const char*, opl::openplugin** plug )
	{
		*plug = new plugin::avformat_plugin;
		return true;
	}
	
	ML_PLUGIN_DECLSPEC void openplugin_destroy_plugin( opl::openplugin* plug )
	{ 
		delete static_cast< plugin::avformat_plugin * >( plug ); 
	}






}



