
// openpluginlib - An plugin interface to openlibraries.

// Copyright (C) 2005-2006 Visual Media FX Ltd.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.

#include <openpluginlib/pl/GL_utility.hpp>

namespace olib { namespace openpluginlib {

std::pair<int, GLenum> pf_to_gl_format( const wstring& pf )
{
	if( pf == L"dxt1" )
	{
		return std::make_pair( -1, GL_COMPRESSED_RGB_S3TC_DXT1_EXT );
	}
	else if( pf == L"dxt3" )
	{
		return std::make_pair( -1, GL_COMPRESSED_RGBA_S3TC_DXT3_EXT );
	}
	else if( pf == L"dxt5" )
	{
		return std::make_pair( -1, GL_COMPRESSED_RGBA_S3TC_DXT5_EXT );
	}
	else if( pf == L"b8g8r8a8" )
	{
		return std::make_pair( 4, GL_BGRA_EXT );
	}
	else if( pf == L"a8b8g8r8" )
	{
		return std::make_pair( 4, GL_ABGR_EXT );
	}
	else if( pf == L"b8g8r8" )
	{
		return std::make_pair( 3, GL_BGR_EXT );
	}
	else if( pf == L"r8g8b8" )
	{
		return std::make_pair( 3, GL_RGB );
	}
	else if( pf == L"r8g8b8a8" )
	{
		return std::make_pair( 4, GL_RGBA );
	}
	else if( pf == L"l8" )
	{
		return std::make_pair( 1, GL_LUMINANCE );
	}
	else if( pf == L"yuv422" )
	{
		if( GLEW_APPLE_ycbcr_422 )
			return std::make_pair( 3, GL_YCBCR_422_APPLE );
	}
	
	return std::make_pair( 0, 0 );
}

bool is_compressed_format( const wstring& pf )
{
	return pf == L"dxt1" || pf == L"dxt3" || pf == L"dxt5";
}

bool is_yuv_format( const wstring& pf )
{
	return pf == L"yuv444" || pf == L"yuv420p";
}

bool texture_target( size_t width, size_t height, GLenum& target, float& tex_w, float& tex_h, bool force_2_0_targets )
{
	if( ( ( GLEW_VERSION_2_0 || GLEW_ARB_texture_non_power_of_two ) && force_2_0_targets ) || ( !( width & ( width - 1 ) ) ) && ( !( height & ( height - 1 ) ) ) )
	{
		target = GL_TEXTURE_2D;
		tex_w = 1.0f, tex_h = 1.0f;
		return true;
	}
	else if( GLEW_ARB_texture_rectangle )
	{
		target = GL_TEXTURE_RECTANGLE_ARB;
		tex_w = static_cast<float>( width ), tex_h = static_cast<float>( height );
		return true;
	}
	else if( GLEW_EXT_texture_rectangle )
	{
		target = GL_TEXTURE_RECTANGLE_EXT;
		tex_w = static_cast<float>( width ), tex_h = static_cast<float>( height );
		return true;
	}
	else if( GLEW_NV_texture_rectangle )
	{
		target = GL_TEXTURE_RECTANGLE_NV;
		tex_w = static_cast<float>( width ), tex_h = static_cast<float>( height );
		return true;
	}

	return false;
}

bool proxy_texture_target( size_t width, size_t height, GLenum& target, bool force_2_0_targets )
{
	if( ( ( GLEW_VERSION_2_0 || GLEW_ARB_texture_non_power_of_two ) && force_2_0_targets ) || ( !( width & ( width - 1 ) ) ) && ( !( height & ( height - 1 ) ) ) )
	{
		target = GL_PROXY_TEXTURE_2D;
		return true;
	}
	else if( GLEW_ARB_texture_rectangle )
	{
		target = GL_PROXY_TEXTURE_RECTANGLE_ARB;
		return true;
	}
	else if( GLEW_EXT_texture_rectangle )
	{
		target = GL_PROXY_TEXTURE_RECTANGLE_EXT;
		return true;
	}
	else if( GLEW_NV_texture_rectangle )
	{
		target = GL_PROXY_TEXTURE_RECTANGLE_NV;
		return true;
	}

	return false;
}

} }
