
// openpluginlib - A plugin interface to openlibraries.

// Copyright (C) 2005-2006 Visual Media FX Ltd.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.

#include "key.hpp"

// boost
#include <boost/functional/hash.hpp>

// std
#include <iostream>
#include <map>

namespace olib { namespace openpluginlib { namespace pcos {

typedef std::map< const char*, std::size_t > string_key_map;
typedef std::map< std::size_t, const char* > key_string_map;

static key_string_map& keyStringMap()
{
    static key_string_map static_keyStringMap;
    return static_keyStringMap;
}

static string_key_map& stringKeyMap()
{
    static string_key_map static_stringKeyMap;
    return static_stringKeyMap;
}

key key::fromString( const char* keyAsString )
{
    if ( !stringKeyMap().count( keyAsString ) )
    {
        boost::hash< const char* > hashFn;
        std::size_t h = hashFn( keyAsString );

        stringKeyMap()[ keyAsString ] = h;
        keyStringMap()[ h ] = keyAsString;
    }

    return key( stringKeyMap()[ keyAsString ] );
}

key::key( std::size_t id )
    : m_id( id )
{}
    
bool key::operator<( const key& k ) const
{
    return m_id < k.m_id;
}

bool key::operator==( const key& k ) const
{
    return m_id == k.m_id;
}

bool key::operator!=( const key& k ) const
{
    return m_id != k.m_id;
}

std::ostream& operator<<( std::ostream& os, const key& k )
{
    os << "key: " << keyStringMap()[ k.m_id ] << ", (" << k.m_id << ")";
    return os;
}

} } }
