
// openpluginlib - A plugin interface to openlibraries.

// Copyright (C) 2005 Visual Media FX Ltd.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.

#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/path.hpp>
#include <boost/regex.hpp>
#include <boost/tokenizer.hpp>

#include <openpluginlib/pl/registry.hpp>
#include <openpluginlib/pl/utf8_utils.hpp>
#include <openpluginlib/pl/opl_importer.hpp>

namespace fs = boost::filesystem;

namespace olib { namespace openpluginlib { namespace detail {

registry*	registry::instance_		 = 0;
bool		registry::destroyed_	 = false;
bool		registry::was_destroyed_ = false;

registry::registry( const string& /*lookup_path*/ )
{
}

registry::~registry( )
{
}

bool registry::insert( const string& lookup_path )
{
	namespace fs = boost::filesystem;

	if( lookup_path.empty( ) )
		return false;

	boost::regex opl_ex( ".*\\.opl", boost::regex::extended | boost::regex::icase );

	typedef boost::tokenizer<boost::char_separator<char> > tokenizer;

#ifdef WIN32
	boost::char_separator<char> sep( ";" );
#else
	boost::char_separator<char> sep( ":" );
#endif

	std::string lp( lookup_path.c_str( ) );

	tokenizer tok( lp.begin( ), lp.end( ), sep );
	for( tokenizer::const_iterator I = tok.begin( ); I != tok.end( ); ++I )
	{
		if( !fs::exists( fs::path( *I, fs::native ) ) || !fs::is_directory( fs::path( *I, fs::native ) ) ) continue;

		fs::directory_iterator end_iter;
		for( fs::directory_iterator dir_iter( fs::path( *I, fs::native ) ); dir_iter != end_iter; ++dir_iter )
		{
			if( boost::regex_match( ( *dir_iter ).string( ), opl_ex ) )
			{
				opl_importer importer;
				importer( fs::path( ( *dir_iter ).string( ), fs::native ) );
				
				std::copy( importer.plugins.begin( ), importer.plugins.end( ), std::inserter( db_, db_.begin( ) ) );
			}
		}
	}

	return true;
}

bool registry::remove( )
{
	return false;
}

void registry::clear( )
{
	db_.clear( );	
}

} } }
